@@ -11,7 +11,7 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from PyQt4.Qsci import QsciScintilla
 
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import cmdcore, qtlib, qscilib
 
 def startProgress(topic, status):
@@ -66,7 +66,7 @@ class ProgressMonitor(QWidget):
 
 
 class ThgStatusBar(QStatusBar):
-    linkActivated = pyqtSignal(QString)
+    linkActivated = pyqtSignal(str)
 
     def __init__(self, parent=None):
         QStatusBar.__init__(self, parent)
@@ -74,9 +74,13 @@ class ThgStatusBar(QStatusBar):
         self.lbl = QLabel()
         self.lbl.linkActivated.connect(self.linkActivated)
         self.addWidget(self.lbl)
+        self._busyrepos = set()
+        self._busypbar = QProgressBar(self, minimum=0, maximum=0)
+        self.addWidget(self._busypbar)
         self.setStyleSheet('QStatusBar::item { border: none }')
+        self._updateBusyProgress()
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def showMessage(self, ustr, error=False):
         self.lbl.setText(ustr)
         if error:
@@ -84,13 +88,29 @@ class ThgStatusBar(QStatusBar):
         else:
             self.lbl.setStyleSheet('')
 
+    def setRepoBusy(self, root, busy):
+        root = unicode(root)
+        if busy:
+            self._busyrepos.add(root)
+        else:
+            self._busyrepos.discard(root)
+        self._updateBusyProgress()
+
+    def _updateBusyProgress(self):
+        # busy indicator is the last option, which is visible only if no
+        # progress information is available
+        visible = bool(self._busyrepos and not self.topics)
+        self._busypbar.setVisible(visible)
+        if visible:
+            self._busypbar.setMaximumSize(150, self.lbl.sizeHint().height())
+
     @pyqtSlot()
     def clearProgress(self):
         keys = self.topics.keys()
         for key in keys:
             self._removeProgress(key)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def clearRepoProgress(self, root):
         root = unicode(root)
         keys = [k for k in self.topics if k[0] == root]
@@ -102,9 +122,10 @@ class ThgStatusBar(QStatusBar):
         self.removeWidget(pm)
         pm.setParent(None)
         del self.topics[key]
+        self._updateBusyProgress()
 
     # TODO: migrate to setProgress() API
-    @pyqtSlot(QString, object, QString, QString, object)
+    @pyqtSlot(str, object, str, str, object)
     def progress(self, topic, pos, item, unit, total, root=None):
         'Progress signal received from repowidget'
         # topic is current operation
@@ -124,6 +145,7 @@ class ThgStatusBar(QStatusBar):
             pm.setMaximumHeight(self.lbl.sizeHint().height())
             self.addWidget(pm)
             self.topics[key] = pm
+            self._updateBusyProgress()
         else:
             pm = self.topics[key]
         if total:
@@ -142,7 +164,7 @@ class ThgStatusBar(QStatusBar):
     def setProgress(self, progress):
         self.progress(*progress)
 
-    @pyqtSlot(unicode, cmdcore.ProgressMessage)
+    @pyqtSlot(str, cmdcore.ProgressMessage)
     def setRepoProgress(self, root, progress):
         self.progress(*(progress + (unicode(root),)))
 
@@ -191,12 +213,12 @@ class LogWidget(qscilib.ScintillaCompat):
             # NOTE: self.setMarkerForegroundColor() doesn't take effect,
             # because it's a *Background* marker.
 
-    @pyqtSlot(unicode, str)
+    @pyqtSlot(str, str)
     def appendLog(self, msg, label):
         """Append log text to the last line; scrolls down to there"""
         self.append(msg)
         self._setmarker(xrange(self.lines() - unicode(msg).count('\n') - 1,
-                               self.lines() - 1), label)
+                               self.lines() - 1), unicode(label))
         self.setCursorPosition(self.lines() - 1, 0)
 
     def _setmarker(self, lines, label):
@@ -205,7 +227,7 @@ class LogWidget(qscilib.ScintillaCompat):
                 self.markerAdd(i, m)
 
     def _markersforlabel(self, label):
-        return iter(self._markers[l] for l in str(label).split()
+        return iter(self._markers[l] for l in label.split()
                     if l in self._markers)
 
     @pyqtSlot()
@@ -311,7 +333,7 @@ class CmdSessionControlWidget(QWidget):
     """Helper widget to implement dialog to run Mercurial commands"""
 
     finished = pyqtSignal(int)
-    linkActivated = pyqtSignal(unicode)
+    linkActivated = pyqtSignal(str)
     logVisibilityChanged = pyqtSignal(bool)
 
     # this won't provide commandFinished signal because the client code
