@@ -13,17 +13,17 @@ from PyQt4.Qsci import QsciScintilla
 
 from mercurial import commands, util
 
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import cmdui
+from tortoisehg.hgqt import cmdui, qtlib
 from tortoisehg.util import hglib
+from tortoisehg.util.i18n import _
 
 class _LogWidgetForConsole(cmdui.LogWidget):
     """Wrapped LogWidget for ConsoleWidget"""
 
-    returnPressed = pyqtSignal(unicode)
+    returnPressed = pyqtSignal(str)
     """Return key pressed when cursor is on prompt line"""
-    historyRequested = pyqtSignal(unicode, int)  # keyword, direction
-    completeRequested = pyqtSignal(unicode)
+    historyRequested = pyqtSignal(str, int)  # keyword, direction
+    completeRequested = pyqtSignal(str)
 
     _prompt = '% '
 
@@ -258,19 +258,19 @@ def _searchhistory(items, text, direction, idx):
         idx += direction
     return None, idx
 
-class ConsoleWidget(QWidget):
+class ConsoleWidget(QWidget, qtlib.TaskWidget):
     """Console to run hg/thg command and show output"""
     closeRequested = pyqtSignal()
 
     def __init__(self, agent, parent=None):
-        super(ConsoleWidget, self).__init__(parent)
+        QWidget.__init__(self, parent)
         self.setLayout(QVBoxLayout())
         self.layout().setContentsMargins(0, 0, 0, 0)
         self._initlogwidget()
         self.setFocusProxy(self._logwidget)
         self._agent = agent
         agent.busyChanged.connect(self._suppressPromptOnBusy)
-        agent.outputReceived.connect(self._logwidget.appendLog)
+        agent.outputReceived.connect(self.appendLog)
         if util.safehasattr(agent, 'displayName'):
             self._logwidget.setPrompt('%s%% ' % agent.displayName())
         self.openPrompt()
@@ -288,7 +288,7 @@ class ConsoleWidget(QWidget):
         for name in ('openPrompt', 'closePrompt', 'clear'):
             setattr(self, name, getattr(self._logwidget, name))
 
-    @pyqtSlot(unicode, int)
+    @pyqtSlot(str, int)
     def historySearch(self, text, direction):
         cmdline, idx = _searchhistory(self._commandHistory, unicode(text),
                                       direction, self._commandIdx)
@@ -350,7 +350,7 @@ class ConsoleWidget(QWidget):
                     matches = [basecmdline % opt[1] for opt in cmdopts]
         return sorted(matches)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def completeCommandText(self, text):
         """Show the list of history or known commands matching the search text
 
@@ -417,9 +417,9 @@ class ConsoleWidget(QWidget):
         text = hglib.tounicode(self._extproc.readAllStandardError().data())
         self._logwidget.appendLog(text, 'ui.warning')
 
-    @pyqtSlot(unicode, str)
+    @pyqtSlot(str, str)
     def appendLog(self, msg, label):
-        """Append log text from another cmdui"""
+        """Append log text to the last line while keeping the prompt line"""
         self._logwidget.clearPrompt()
         try:
             self._logwidget.appendLog(msg, label)
@@ -446,7 +446,7 @@ class ConsoleWidget(QWidget):
         else:
             self.openPrompt()
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _runcommand(self, cmdline):
         cmdline = unicode(cmdline)
         self._commandIdx = 0
@@ -479,7 +479,7 @@ class ConsoleWidget(QWidget):
             raise ValueError(_('command parse error: %s') % e)
         for e in args:
             e = util.expandpath(e).decode('utf-8')
-            if util.any(c in e for c in '*?[]'):
+            if any(c in e for c in '*?[]'):
                 expanded = glob.glob(os.path.join(self._workingDirectory(), e))
                 if not expanded:
                     raise ValueError(_('no matches found: %s') % e)
@@ -522,6 +522,7 @@ class ConsoleWidget(QWidget):
         'exit':  _cmd_exit,
         }
 
+
 class LogDockWidget(QDockWidget):
 
     def __init__(self, repomanager, cmdagent, parent=None):
@@ -530,7 +531,7 @@ class LogDockWidget(QDockWidget):
         self.setFeatures(QDockWidget.DockWidgetClosable |
                          QDockWidget.DockWidgetMovable  |
                          QDockWidget.DockWidgetFloatable)
-        self.setWindowTitle(_('Output Log'))
+        self.setWindowTitle(_('Console'))
         # Not enabled until we have a way to make it configurable
         #self.setWindowFlags(Qt.Drawer)
         self.dockLocationChanged.connect(self._updateTitleBarStyle)
@@ -545,12 +546,10 @@ class LogDockWidget(QDockWidget):
         for root in self._repomanager.repoRootPaths():
             self._createConsoleFor(root)
 
-        # move focus only when console is activated by keyboard/mouse operation
-        self.toggleViewAction().triggered.connect(self._setFocusOnToggleView)
-
     def setCurrentRepoRoot(self, root):
         w = self._findConsoleFor(root)
         self._consoles.setCurrentWidget(w)
+        self.setFocusProxy(w)
 
     def _findConsoleFor(self, root):
         for i in xrange(self._consoles.count()):
@@ -565,35 +564,20 @@ class LogDockWidget(QDockWidget):
         self._consoles.addWidget(w)
         return w
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _createConsoleFor(self, root):
         root = unicode(root)
         repoagent = self._repomanager.repoAgent(root)
         assert repoagent
         self._createConsole(repoagent)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _destroyConsoleFor(self, root):
         root = unicode(root)
         w = self._findConsoleFor(root)
         self._consoles.removeWidget(w)
         w.setParent(None)
 
-    @pyqtSlot()
-    def clear(self):
-        w = self._consoles.currentWidget()
-        w.clear()
-
-    def appendLog(self, msg, label, reporoot=None):
-        w = self._findConsoleFor(reporoot)
-        w.appendLog(msg, label)
-
-    @pyqtSlot(bool)
-    def _setFocusOnToggleView(self, visible):
-        if visible:
-            w = self._consoles.currentWidget()
-            w.setFocus()
-
     def setVisible(self, visible):
         super(LogDockWidget, self).setVisible(visible)
         if visible:
